# Generated by Django 3.1.7 on 2021-11-16 13:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtailcore", "0062_comment_models_and_pagesubscription"),
        ("wagtail_trash", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="trashcan",
            options={"verbose_name": "Trash Can", "verbose_name_plural": "Trash Cans"},
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="data",
            field=models.TextField(blank=True, verbose_name="Data"),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="page",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="wagtailcore.page",
                verbose_name="Page",
            ),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="parent",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailcore.page",
                verbose_name="Parent",
            ),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="time_recycled",
            field=models.DateTimeField(auto_now_add=True, verbose_name="Time Recycled"),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="user",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to=settings.AUTH_USER_MODEL,
                verbose_name="User",
            ),
        ),
    ]
